<?php
namespace Service;
use \Util\Exception;

class GeneralFunctions {

	protected $entityManager;
	protected $notaFiscal;
	protected $impressaoNFCE;
	protected $caixaApi;
	protected $databaseUtil;
	protected $impressaoSAT;
	protected $vendaAPI;
	protected $impressaoPedido;
	protected $impressaoPedidoSmart;
	protected $util;
	protected $paymentService;
	protected $billService;


	public function __construct(
		\Doctrine\ORM\EntityManager $entityManager,
		\Odhen\API\Service\NotaFiscal $notaFiscal,
		\Odhen\API\Service\ImpressaoNFCE $impressaoNFCE,
		\Odhen\API\Service\Caixa $caixaApi, 
		\Odhen\API\Util\DataBase $databaseUtil,
		\Odhen\API\Service\ImpressaoSAT $impressaoSAT, 
		\Odhen\API\Service\Venda $vendaAPI, 
		\Odhen\API\Service\ImpressaoPedido $impressaoPedido, 
		\Odhen\API\Service\ImpressaoPedidoSmart $impressaoPedidoSmart, 
		\Util\Util $util,
		\Service\Payment $paymentService,
	    \Service\Bill $billService){

		$this->entityManager = $entityManager;
		$this->notaFiscal = $notaFiscal;
		$this->impressaoNFCE = $impressaoNFCE;
		$this->caixaApi = $caixaApi;
		$this->databaseUtil = $databaseUtil;
		$this->impressaoSAT = $impressaoSAT;
		$this->vendaAPI = $vendaAPI;
		$this->impressaoPedido = $impressaoPedido;
		$this->impressaoPedidoSmart = $impressaoPedidoSmart;
		$this->util = $util;
		$this->paymentService = $paymentService;
		$this->billService  = $billService;
	}

	public function reprintSaleCoupon($session, $reprintType, $NRNOTAFISCALCE, $informaEmail, $consumerEmail, $CDCAIXA){
		$CDFILIAL = $session['CDFILIAL'];
		$NRORG = $session['NRORG'];
		$DTEMISSAONFCE = new \DateTime();
		$DTEMISSAONFCE = $this->databaseUtil->databaseIsOracle() ? $DTEMISSAONFCE->format('Y-m-d') : $DTEMISSAONFCE;

		$params = array(
			'CDFILIAL' => $CDFILIAL,
			'CDCAIXA' => $CDCAIXA,
			'NRORG' => $NRORG,
			'IDTPEMISVEND' =>  'N',
			'DTEMISSAONFCE' => $DTEMISSAONFCE
		);

		if ($session['IDTPEMISSAOFOS'] === 'FNC') {
			$CDSERIENFCE = $this->entityManager->getConnection()->fetchAssoc('GET_SERIE_NFCE', $params);
			$params['CDSERIENFCE'] = !empty($CDSERIENFCE['CDSERIECX']) ? $CDSERIENFCE['CDSERIECX'] : $CDSERIENFCE['CDCAIXA'];
		} else {
			$params['CDSERIENFCE'] = 'SAT';
			$params['IDTPEMISVEND'] = 'S';
		}

		$type = $this->databaseUtil->databaseIsOracle() ? array() : array('DTEMISSAONFCE' => \Doctrine\DBAL\TypeS\Type::DATETIME);

		if ($reprintType == 'U'){
			$saleCode = $this->entityManager->getConnection()->fetchAssoc('GET_ULTIMA_VENDA_FISCAL', $params, $type);
			$NRNOTAFISCALCE = !empty($saleCode['NRNOTAFISCALCE']) ? $saleCode['NRNOTAFISCALCE'] : null;
		}
		$params['NRNOTAFISCALCE'] = $NRNOTAFISCALCE;

		// Parametro IDENVCUPOMEMAIL controla se a reimpressão vai trazer todas as vendas realizadas ou somente as do dia.
		if($this->databaseUtil->databaseIsOracle()) {
			$params['DTEMISSAONFCE'] = ($session['IDENVCUPOMEMAIL'] == 'S') ? '%%' : (new \DateTime($DTEMISSAONFCE))->format('Y-m-d');

		} else {
			$params['DTEMISSAONFCE'] = ($session['IDENVCUPOMEMAIL'] == 'S') ? '%%' : $DTEMISSAONFCE->format('Y-m-d');
		}
		$venda = $this->entityManager->getConnection()->fetchAssoc('GET_VENDA', $params);
		if (!empty($venda)){
			$params['NRSEQVENDA'] = $venda['NRSEQVENDA'];
			$NRMESA = $venda['NRMESA'];

			$FIDELITYVALUE = $this->getDiscountFidelity($CDFILIAL, $CDCAIXA, $venda['CDCLIENTE'], $venda['CDCONSUMIDOR'], '010', $venda['NRSEQVENDA']);
			$VRDESCCASHUDS = $this->getVRDESCCASHUDS($CDFILIAL, $CDCAIXA, $venda['NRSEQVENDA']);
			$FIDELITYVALUE = $FIDELITYVALUE+$VRDESCCASHUDS;
			if ($session['IDTPEMISSAOFOS'] === 'FNC') {
				$params['IDTPAMBNFCE'] = $this->entityManager->getConnection()->fetchAssoc('SQL_FILIAL_DETAILS', array('CDFILIAL' => $CDFILIAL))['IDAMBTRABNFCE'];
				$nfceInfo = array(
					'IDSTATUSNFCE' => $venda['IDSTATUSNFCE'],
					'tpAmb' => $params['IDTPAMBNFCE'],
					'nrseqvenda' => $venda['NRSEQVENDA'],
					'DTHRPROTOCONFCE' => new \DateTime($venda['DTHRPROTOCONFCE']),
					'VRTOTTRIBIBPT' => $this->entityManager->getConnection()->fetchAssoc('GET_VRTOTTRIBIBPT_ITVENDAIMPOS', $params)['VRTOTTRIBIBPT'],
					'nrAcessoNFCE' => $venda['NRACESSONFCE']
				);

				$endeFiliResult = $this->notaFiscal->validaEnderecoFilial($CDFILIAL, $NRORG);
				$dadosEmitente = $this->notaFiscal->validaDadosEmitenteXML($CDFILIAL, $NRORG);
				$filialInfo = array(
					'CDFILIAL' 			=> $CDFILIAL,
					'CDLOJA' 			=> $session['CDLOJA'],
					'CDCAIXA' 			=> $CDCAIXA,
					'CDCAIXA_IMPRESSAO' => $session['CDCAIXA'],
					'CDCAIXA_CUPOM' 	=> $CDCAIXA,
					'SGESTADO' 			=> $endeFiliResult['dadosEndereco']['SGESTADO'],
					'NRMESA'   			=> $NRMESA,
					'NRORG'	   			=> $NRORG
				);
				$filialInfo = array_merge($filialInfo, $dadosEmitente['dadosEmitente'], $endeFiliResult['dadosEndereco']);
				$nrseqvenda = array('nrseqvenda' => $venda['NRSEQVENDA']);
				$productsNFCE = $this->notaFiscal->getInfoProducts($params, $nrseqvenda);
				$infoConsumer = $this->notaFiscal->getInfoConsumer($params, $nrseqvenda);

                //verifica se o parâmetro IDENVCUPOMEMAIL foi selecionado, ou seja, envia cupom por email
				if ($session['IDENVCUPOMEMAIL'] == 'S') {
					$DSEMAILCONS = $informaEmail ? $consumerEmail : null;
	                $IDSITENVCUPOM = !empty($DSEMAILCONS) ? 'A' : 'N';
					$params = array(
						'NRORG'	   	=> $NRORG,
						'NRSEQVENDA'=>$venda['NRSEQVENDA'],
						'CDFILIAL' 	=> $CDFILIAL,
						'CDCAIXA' 	=> $CDCAIXA,
						'DSEMAILCONS'=> $DSEMAILCONS,
						'IDSITENVCUPOM'=> $IDSITENVCUPOM
					);
					$this->entityManager->getConnection()->executeQuery("UPDATE_VENDA", $params);
					$result = array(
						'error' =>  false,
						'message' => '',
						'dadosImpressao' => array(),
						'paramsImpressora' => array(),
						'errPainelSenha' => ''
					);
				} else {
					$result = $this->impressaoNFCE->imprimeDanfeNFCE($nfceInfo, $filialInfo, $productsNFCE, $infoConsumer, false, $FIDELITYVALUE, null, null, null);
				}
			} else {
                $ITEMVENDA = $this->entityManager->getConnection()->fetchAll("BUSCA_PRODUTOS_SAT", $params);
                $impostos = $this->vendaAPI->calculaImpostoParaImpressao($NRORG, $CDFILIAL, $CDCAIXA, $ITEMVENDA, $venda['VRTOTVENDA'], $venda['VRDESCVENDA'], $session['CDLOJA'], $venda['CDCLIENTE'], $venda['CDCONSUMIDOR'], $venda['VRTXSEVENDA']);
				$VRTOTTRIBIBPT = $impostos['VRTOTTRIBIBPT'];
				$impostoEstadual = $impostos['impostoEstadual'];

				// verifica se o parâmetro IDENVCUPOMEMAIL foi selecionado, ou seja, envia cupom por email
				if ($session['IDENVCUPOMEMAIL'] == 'S') {

					$DSEMAILCONS = $informaEmail ? $consumerEmail : null;
	                $IDSITENVCUPOM = !empty($DSEMAILCONS) ? 'A' : 'N';

					$params = array(
						'NRORG'	   	=> $NRORG,
						'NRSEQVENDA'=>$venda['NRSEQVENDA'],
						'CDFILIAL' 	=> $CDFILIAL,
						'CDCAIXA' 	=> $CDCAIXA,
						'DSEMAILCONS'=> $DSEMAILCONS,
						'IDSITENVCUPOM'=> $IDSITENVCUPOM
					);
					$this->entityManager->getConnection()->executeQuery("UPDATE_VENDA", $params);

					$result = array(
						'error' =>  false,
						'message' => '',
						'dadosImpressao' => array(),
						'paramsImpressora' => array(),
						'errPainelSenha' => ''
					);
				} else {
					$result = $this->impressaoSAT->imprimeCupomNF($CDFILIAL, $session['CDCAIXA'], $NRORG, $venda['NRSEQVENDA'], $NRNOTAFISCALCE, $venda['CDSERIESAT'], $DTEMISSAONFCE, $venda['NRACESSONFCE'], $venda['NRINSCRCONS'], $venda['NMCONSVEND'], $venda['CDSENHAPED'], $venda['DSQRCODENFCE'], $VRTOTTRIBIBPT, $impostoEstadual, $FIDELITYVALUE, $ITEMVENDA, $NRMESA, $venda['CDCAIXA'], null, null, null);
				}
			}
		} else {
			$result = array(
				'error' => true,
				'message' => 'Operação bloqueada. Venda não encontrada.'
			);
		}

		return $result;
	}

	private function buildObsItemVenda($group, array $obs, $obsDigita) {
		$obsNames = $this->paymentService->getOcorTxts($group, $obs);

		if (!empty($obsDigita)){
			$obsNames[] = $obsDigita;
		}

		return implode('; ', $obsNames);
	}

	private function handleProducts($session, $ITEMVENDA, $CDFILIAL, $CDLOJA, $CDCLIENTE, $CDCONSUMIDOR, $CDVENDEDOR) {
		$handledProducts = array();
		$ordemImp = 1;
        $voucherUsage = array();
		foreach ($ITEMVENDA as $ITEM){
			$newProduct = array();
			$ITEM = self::fillEmptyProperties($ITEM);
			$newProduct['NMPRODUTO'] = $ITEM['NMPRODUTO'];
			$newProduct['NMPRODPROMOCAO'] = $ITEM['NMPRODUTO'];
			$newProduct['CDPRODUTO'] = $ITEM['CDPRODUTO'];
			$newProduct['QTPRODVEND'] = $ITEM['QTPRODCOMVEN'];
			$newProduct['QTPRODCOMVEN'] = $ITEM['QTPRODCOMVEN'];
			$newProduct['VRUNITVEND'] = $ITEM['VRPRECCOMVEN'];
            $newProduct['VRUNITVENDCL'] = $ITEM['VRPRECCLCOMVEN'];
            //$newProduct['REALSUBSIDY'] = $ITEM['REALSUBSIDY'];
			$newProduct['NRLUGARMESA'] = '1';
			$newProduct['VRDESITVEND'] = ($ITEM['VRDESCCOMVEN'] > $ITEM['VRPRECCOMVEN']) ? $ITEM['VRPRECCOMVEN'] : $ITEM['VRDESCCOMVEN'];
			$newProduct['VRACRITVEND'] = $ITEM['VRACRCOMVEN'];
			$newProduct['IDSITUITEM'] = 'A';
			$newProduct['IDTIPOITEM'] = null;
			$newProduct['ORDEMIMP'] = $ordemImp;
			$newProduct['IDTIPOCOMPPROD'] = $ITEM['IDTIPOCOMPPROD'];
			$newProduct['IDIMPPROMOCAO'] = ($ITEM['IDTIPOCOMPPROD'] == '3' || $ITEM['IDTIPOCOMPPROD'] == 'C') ? 'S' : 'N';
			$newProduct['IDIMPPRODUTO'] = $ITEM['IDIMPPRODUTO'];
			$newProduct['TXPRODCOMVEN'] = !empty($ITEM['TXPRODCOMVEN']) ? $ITEM['TXPRODCOMVEN'] : null;
			$newProduct['CDSUPERVISOR'] = null;
			$ITEM['CDOCORR'] = $ITEM['CDOCORR'] != null? $ITEM['CDOCORR'] : array();
			$newProduct['DSOBSITEMVENDA'] = $this->buildObsItemVenda($session['CDGRPOCORPED'], $ITEM['CDOCORR'], $ITEM['DSOCORR_CUSTOM']);
			$newProduct['DSOBSPEDDIGITA'] = $ITEM['DSOCORR_CUSTOM'];
			$newProduct['DTHRPRODCANVEN'] = $ITEM['DTHRPRODCANVEN'];
			$newProduct['OBSERVACOES'] = array_map(function($obs) use ($session) {
				return array(
					'CDOCORR' => $obs,
					'CDGRPOCOR' => $session['CDGRPOCORPED']
				);
			}, $ITEM['CDOCORR']);
			$newProduct['CDGRPOCOR'] = null;
            $newProduct['ATRASOPROD'] = $session['NRATRAPADRAO'];
            $newProduct['IDORIGEMVENDA'] = null;
			$newProduct['DSOBSDESCIT'] = null;
			$newProduct['CDGRPOCORDESCIT'] = null;
			$newProduct['CDVENDEDOR'] = $CDVENDEDOR;
			$ITEM['PRODUTOS'] = isset($ITEM['PRODUTOS'])?$ITEM['PRODUTOS']:array();
			
			if (array_key_exists('CDPRODPROMOCAO', $ITEM)) {
				$newProduct['CDPRODPROMOCAO'] = $ITEM['CDPRODPROMOCAO'];
			} else {
				$newProduct['CDPRODPROMOCAO'] = empty($ITEM['PRODUTOS']) ? null : $ITEM['CDPRODUTO'];
			}

			$comboItems = self::montaItensdoCombo($ITEM,$session,$ordemImp,$CDVENDEDOR,$voucherUsage);

            $newProduct['VOUCHER'] = $this->checkVoucherUsage($ITEM, $voucherUsage);
            if (!array_key_exists('IMPRIMESIM', $ITEM)) $ITEM['IMPRIMESIM'] = false; // Correção temporária.
			$newProduct['IDIMPPRGRPROMO'] = !!$ITEM['IMPRIMESIM'] ? 'S' : 'N';
            $newProduct['IDOBSERVACAOAD'] = 'N';

            $productDetails = $this->vendaAPI->getProductDetails($newProduct['CDPRODUTO']);
			if (empty($productDetails)){
    			$newProduct['IDPESAPROD'] = 'N';
    			$newProduct['IDIMPRODUVEZ'] = 'N';
			} else {
    			$newProduct['IDPESAPROD'] = $productDetails['IDPESAPROD'];
    			$newProduct['IDIMPRODUVEZ'] = $productDetails['IDIMPRODUVEZ'];
			}

            if (!empty($ITEM['CAMPANHA'])){
                $newProduct['CDCAMPCOMPGANHE'] = $ITEM['CAMPANHA'];
                if ($this->util->databaseIsOracle()){
                    $newProduct['DTINIVGCAMPCG'] = \DateTime::createFromFormat('Y-m-d H:i:s', $ITEM['DTINIVGCAMPCG']);
                }
                else {
                    $newProduct['DTINIVGCAMPCG'] = \DateTime::createFromFormat('Y-m-d H:i:s.u', $ITEM['DTINIVGCAMPCG']);
                }
            }
            else {
                $newProduct['CDCAMPCOMPGANHE'] = null;
                $newProduct['DTINIVGCAMPCG'] = null;
            }

			if (!empty($comboItems)){
				// rotina específica do Madero
				$this->util->validaDescontoDiferenciado($CDFILIAL, $ITEM['CDPRODUTO'], $comboItems, 'VRDESITVEND');
			}

			$newProduct['itensCombo'] = $comboItems;
			$newProduct['ORIGINALQUANT'] = null;
			$newProduct['COMBOQUANT'] = $ITEM['IDTIPOCOMPPROD'] == 'C' ? $ITEM['COMBOQUANT'] : null;

			array_push($handledProducts, $newProduct);

			$ordemImp++;
		}
			
		$handledProducts = self::checkCombos($handledProducts);
		
		return $handledProducts;
	}

	private function montaItensdoCombo($ITEM,$session,$ordemImp,$CDVENDEDOR,$voucherUsage){
		$ordemImpPromo = 0.01;
		$comboItems = array();
		$ITEM['PRODUTOS'] = isset($ITEM['PRODUTOS'])?$ITEM['PRODUTOS']:array();
		foreach ($ITEM['PRODUTOS'] as $currentComboItem) {
			if($currentComboItem==null){continue;}
			$newItem = array();
            $newItem['CDPRODPROMOCAO'] = $ITEM['IDTIPOCOMPPROD'] == 'C' ? null : $ITEM['CDPRODUTO'];
			$newItem['CDPRODUTO'] = $currentComboItem['CDPRODUTO'];
			$newItem['NMPRODUTO'] = $currentComboItem['DSBUTTON'];
			$newItem['QTPRODVEND'] = $currentComboItem['QTPRODCOMVEN'] * $ITEM['QTPRODCOMVEN'];
			$newItem['QTPRODCOMVEN'] = $currentComboItem['QTPRODCOMVEN'] * $ITEM['QTPRODCOMVEN'];
			$newItem['VRUNITVEND'] = $currentComboItem['PRECO'];
            $newItem['VRUNITVENDCL'] = $currentComboItem['VRPRECITEMCL'];
            $newItem['REALSUBSIDY'] = $currentComboItem['REALSUBSIDY'];
			$newItem['VRDESITVEND'] = ($currentComboItem['VRDESITVEND'] > $currentComboItem['PRECO']) ? $currentComboItem['PRECO'] : $currentComboItem['VRDESITVEND'];
			$newItem['NRLUGARMESA'] = '1';
			$newItem['ATRASOPROD'] = $currentComboItem['ATRASOPROD'] == 'Y' ? $session['NRATRAPADRAO'] : null;
            $newItem['IDORIGEMVENDA'] = ($currentComboItem['TOGO'] == 'Y' && $session['IDCTRLPEDVIAGEM'] == 'S') ? 'TGO_BAL' : null;
			$newItem['ORDEMIMP'] = $ordemImp + $ordemImpPromo;
			$newItem['IDTIPOCOMPPROD'] = '1';
			$newItem['IDIMPPROMOCAO'] = $ITEM['IDTIPOCOMPPROD'] == 'C' ? 'N' : 'X';
			$newItem['VRACRITVEND'] = $currentComboItem['VRACRITVEND'];
			$newItem['IDSITUITEM'] = 'A';
			$newItem['IDTIPOITEM'] = null;
			$newItem['TXPRODCOMVEN'] = !empty($currentComboItem['TXPRODCOMVEN']) ? $currentComboItem['TXPRODCOMVEN'] : null;
			$newItem['DSOBSPEDDIGITA'] = $currentComboItem['DSOCORR_CUSTOM'];
			$newItem['DSOBSITEMVENDA'] = $this->buildObsItemVenda($session['CDGRPOCORPED'], $currentComboItem['CDOCORR'], $currentComboItem['DSOCORR_CUSTOM']);
			$newItem['OBSERVACOES'] = array_map(function($obs) use ($session) {
				return array(
					'CDOCORR' => $obs,
					'CDGRPOCOR' => $session['CDGRPOCORPED']
				);
			}, $currentComboItem['CDOCORR']);
			$newItem['IDAPLICADESCPR'] = $currentComboItem['IDAPLICADESCPR'];
			$newItem['IDPERVALORDES'] = $currentComboItem['IDPERVALORDES'];
			$newItem['CDGRPOCOR'] = null;
			$newItem['DSOBSDESCIT'] = null;
			$newItem['CDGRPOCORDESCIT'] = null;
			$newItem['CDVENDEDOR'] = $CDVENDEDOR;
			$newItem['VOUCHER'] = $this->checkVoucherUsage($currentComboItem, $voucherUsage);
			if($ITEM['IDIMPPRODUTO'] == '2'){
				$newItem['IDIMPPRGRPROMO'] = 'N';
			} else {
				$newItem['IDIMPPRGRPROMO'] = $currentComboItem['IDIMPGRPROMO'];
			}

            $newItem['IDOBSERVACAOAD'] = 'N';

            $productDetails = $this->vendaAPI->getProductDetails($newItem['CDPRODUTO']);
            $newItem['IDPESAPROD'] = $productDetails['IDPESAPROD'];
            $newItem['IDIMPRODUVEZ'] = $productDetails['IDIMPRODUVEZ'];

            $newItem['CDCAMPCOMPGANHE'] = null;
            $newItem['DTINIVGCAMPCG'] = null;

			$currentComboItemdoCombo = self::montaItensdoCombo($currentComboItem,$session,$ordemImp,$CDVENDEDOR,$voucherUsage);
			$newItem['itensCombo'] = $currentComboItemdoCombo;

			$newItem['ORIGINALQUANT'] = $ITEM['IDTIPOCOMPPROD'] == 'C' ? $currentComboItem['ORIGINALQUANT'] : null;
			$newItem['COMBOQUANT'] = $ITEM['IDTIPOCOMPPROD'] == 'C' ? $ITEM['COMBOQUANT'] : null;

			$newItem['NMPRODPROMOCAO'] = null;
			if($newItem['IDIMPRODUVEZ']=='S'){
					if(isset($ITEM['DSBUTTON'])){
						$newItem['NMPRODPROMOCAO'] = $ITEM['DSBUTTON'];
					} else if(isset($ITEM['NMPRODUTO'])){
						$newItem['NMPRODPROMOCAO'] = $ITEM['NMPRODUTO'];
					} else if(isset($ITEM['NMPRODPROMOCAO'])){
						$newItem['NMPRODPROMOCAO'] = $ITEM['NMPRODPROMOCAO'];
					}
			}
			array_push($comboItems, $newItem);

			$ordemImpPromo += 0.01;
		}
		
		$comboItems = self::checkCombos($comboItems);
		
		
		return $comboItems;
	}

	private function checkCombos($comboItems){
		$resultCombo = array();
		$result = array();
		$validaProdutoFora = 0;

		foreach($comboItems as $produto){
			if(!empty($produto['itensCombo'])){
				if (($produto['IDTIPOCOMPPROD'] == '3' && $produto['IDIMPPRODUTO'] == '2') || $produto['IDTIPOCOMPPROD'] == 'C'){
					foreach($produto['itensCombo'] as $prodCombo){
						array_push($resultCombo, $prodCombo);
					}
				}
			}
			$result = array_merge($result, $resultCombo);
		}

		if(empty($result)){
			$result = $comboItems;
		}

		return $result;
	}

    private function checkVoucherUsage($product, $voucherUsage){
        if (empty($product['VOUCHER'])) return null;
        else {
            $voucher = $this->paymentService->validateVoucher($product['CDPRODUTO'], null, null, $product['VOUCHER']['CDCUPOMDESCFOS']);
            // Verifica se o voucher já foi utilizado neste pedido.
            if (in_array($product['VOUCHER']['CDCUPOMDESCFOS'], $voucherUsage)){
                throw new \Exception("O voucher " . $product['VOUCHER']['CDCUPOMDESCFOS'] . " é de uso único, mas está sendo aplicado em mais de um produto. Certifique-se de que apenas um produto esteja com este voucher aplicado.");
            }
            else {
                array_push($voucherUsage, $product['VOUCHER']['CDCUPOMDESCFOS']);
            }
            return $voucher;
        }
    }

	private function fillEmptyProperties($item) {

		if (!isset($item['CDOCORR'])) {
			$item['CDOCORR'] = null;
		}
		if (!isset($item['DSOCORR_CUSTOM'])) {
			$item['DSOCORR_CUSTOM'] = null;
		}
		if (!isset($item['PRODUTOS'])) {
			$item['PRODUTOS'] = array();
		}
		if(!isset($item['DTHRPRODCANVEN'])){
			$item['DTHRPRODCANVEN'] = null;
		}
		return $item;
	}


	public function reprintPedido($session, $CDCAIXA, $CDFILIAL, $CDLOJA, $CDVENDEDOR, $IDSENHACUP, $IDIMPPEDPRCIGE, $NRORG){

		$params = array(
			'CDCAIXA' 	=> $CDCAIXA
		);
		$ultPedido = $this->entityManager->getConnection()->fetchAssoc("GET_HORA_ULTIMO_PEDIDO", $params);
		$dataUltPed = str_split($ultPedido['DTINCLUSAO'],16);
		$nrseqvendaUltPed = $ultPedido['NRVENDAREST'];
	
		$params = array(
			'CDCAIXA' 	  => $CDCAIXA,
			'DTINCLUSAO'  => $dataUltPed[0].":00.000",
			'NRVENDAREST' => $nrseqvendaUltPed
		);

		$types = array(
            'DTINCLUSAO' => \Doctrine\DBAL\Types\Type::DATETIME
        );

		$ultPedido = $this->entityManager->getConnection()->fetchAll("GET_ULTIMO_PEDIDO", $params);
		$CDSENHAPED = $ultPedido[0]['CDSENHAPED'];
		$nrComanda = $ultPedido[0]["NRCOMANDA"];
		$valComanda = $this->billService->dadosComanda($CDFILIAL, $nrComanda, $nrseqvendaUltPed, $CDLOJA);

		$nrMesa = null;
		$dsComanda = null;
		$cliente = null;
		$consumidor = null;
		$ORIGEM = 'B';

		if(!empty($valComanda)){
			$nrMesa = str_pad($valComanda['NRMESA'], 4, '0', STR_PAD_LEFT);
			$dsComanda = $valComanda['DSCOMANDA'];
			$cliente = $valComanda['CDCLIENTE'];
			$consumidor = $valComanda['CDCONSUMIDOR'];
			$ORIGEM = $valComanda['IDORGCMDVENDA'];
			if (strpos($ORIGEM, 'MES') !== false){
				$ORIGEM = 'M';
			} else if (strpos($ORIGEM, 'CMD') !== false){
				$ORIGEM = 'C';
			}
		}		

		$ultPedido = $this->handleProducts($session, $ultPedido, $CDFILIAL, $CDLOJA, $cliente, $consumidor, $CDVENDEDOR);
			
		$params = array(
			'CDCAIXA' => $CDCAIXA,
			'CDFILIAL' => $CDFILIAL,
			'CDLOJA' => $CDLOJA
		);

		$resultImpressao = $this->impressaoPedido->imprimePedido(
			$CDFILIAL,
			$CDLOJA,
			$ultPedido,
			$CDVENDEDOR,
			$dsComanda,
			$nrMesa,
			$CDSENHAPED,
			$ORIGEM,
			'',
			false,
			true,
			'',
			$CDCAIXA,
			$IDIMPPEDPRCIGE,
			null,
			'S'//reimpressao
		);

			/*$resultImpressao['PedidoSmart'] = $this->impressaoPedidoSmart->imprimePedidoSmart(
				$CDFILIAL,
				$CDLOJA,
				$ultPedido,
				$CDVENDEDOR,
				$dsComanda,
				$nrMesa,
				$CDSENHAPED,
				$ORIGEM,
				'',
				false,
				true,
				$CDCAIXA,
				$nrseqvendaUltPed,
				$NRORG,
				'S'//reimpressao
			);*/

		$messages = array();
		if ($resultImpressao['error']) {
			$result['error'] = true;
			$result['message'] = '';
			if (!in_array($resultImpressao['message'], $messages)) {
				array_push($messages, $resultImpressao['message']);
				$result['message'] .= $resultImpressao['message'] . ' ';
			}
		} else {
			$result['error'] = false;
		}
		
		if(isset($resultImpressao['paramsImpressora'])){
            $result['paramsImpressora'] = $resultImpressao['paramsImpressora'];
		}
		if(isset($resultImpressao[0]['text'])){
            $result['text'] = $resultImpressao[0]['text'];
		}

		return $result;
	}

	public function selectUnblockedProducts($session, $filter, $FIRST, $LAST){
		$CDFILIAL = $session['CDFILIAL'];
		$NRORG = $session['NRORG'];
        $FILIALVIGENCIA = $session['FILIALVIGENCIA'];
		$NRCONFTELA = $session['NRCONFTELA'];
        $DTINIVIGENCIA = new \DateTime($session['DTINIVIGENCIA']);
		$CDLOJA = $session['CDLOJA'];

		$params = array(
			'CDFILIAL' => $CDFILIAL,
			'NRORG' => $NRORG,
            'FILIALVIGENCIA' => $FILIALVIGENCIA,
			'NRCONFTELA' => $NRCONFTELA,
            'DTINIVIGENCIA' => $DTINIVIGENCIA,
			'CDLOJA' => $CDLOJA,
			'FILTER' => $filter['value'],
			'FIRST' => $FIRST,
			'LAST' => $LAST
		);
        $types = array(
            'DTINIVIGENCIA' => \Doctrine\DBAL\Types\Type::DATETIME
        );
		return $this->entityManager->getConnection()->fetchAll('GET_PRODUTOS_DESBLOQUEADOS', $params, $types);
	}

	public function selectBlockedProducts($session, $filter, $FIRST, $LAST){
		$CDFILIAL = $session['CDFILIAL'];
		$NRORG = $session['NRORG'];
        $FILIALVIGENCIA = $session['FILIALVIGENCIA'];
		$NRCONFTELA = $session['NRCONFTELA'];
        $DTINIVIGENCIA = new \DateTime($session['DTINIVIGENCIA']);
		$CDLOJA = $session['CDLOJA'];
		$CDOPERADOR = $session['CDOPERADOR'];

		$params = array(
			'CDFILIAL' => $CDFILIAL,
			'NRORG' => $NRORG,
            'FILIALVIGENCIA' => $FILIALVIGENCIA,
			'NRCONFTELA' => $NRCONFTELA,
            'DTINIVIGENCIA' => $DTINIVIGENCIA,
			'CDLOJA' => $CDLOJA,
			'CDOPERADOR' => $CDOPERADOR,
			'FILTER' => $filter['value'],
			'FIRST' => $FIRST,
			'LAST' => $LAST
		);
        $types = array(
            'DTINIVIGENCIA' => \Doctrine\DBAL\Types\Type::DATETIME
        );
		return $this->entityManager->getConnection()->fetchAll('GET_PRODUTOS_BLOQUEADOS', $params, $types);
	}

	public function blockProducts($session, $arrProducts){
		try {
			$this->entityManager->getConnection()->beginTransaction();

            $arrProducts = array_unique($arrProducts);

			$params = array(
				'CDFILIAL' => $session['CDFILIAL'],
				'CDLOJA' =>  $session['CDLOJA'],
				'CDPRODUTO' => '',
				'DTHRBLOQUEIO' => new \DateTime('NOW'),
				'CDOPERADOR' => $session['CDOPERADOR']
			);
			$types = array(
				'DTHRBLOQUEIO' => \Doctrine\DBAL\TypeS\Type::DATETIME
			);

			foreach ($arrProducts as $product){
				$params['CDPRODUTO'] = $product;
				$this->entityManager->getConnection()->executeQuery('INSERT_PRODUTOS_BLOQUEADOS', $params, $types);
			}

			$this->entityManager->commit();

			return array('error' => false);

		} catch(\Exception $e){
			Exception::logException($e);

			$this->entityManager->rollback();

			throw new \Exception ($e->getMessage(), 1);
		}
	}

	public function unblockProducts($session, $arrProducts){
		try {
			$params = array(
				'CDFILIAL' => $session['CDFILIAL'],
				'CDLOJA' =>  $session['CDLOJA'],
				'CDPRODUTO' => array_values($arrProducts),
				'CDOPERADOR' => $session['CDOPERADOR']
			);
			$types = array(
				'CDPRODUTO' => \Doctrine\DBAL\Connection::PARAM_STR_ARRAY
			);

			$this->entityManager->getConnection()->executeQuery('DELETE_PRODUTOS_BLOQUEADOS', $params, $types);

			return array('error' => false);

		} catch(\Exception $e){
			Exception::logException($e);

			throw new \Exception ($e->getMessage(), 1);
		}
	}

	public function impressaoLeituraX($session, &$dadosImpressao){
		$result = $this->caixaApi->imprimeLeituraX($session['CDFILIAL'], null, $session['CDCAIXA'], $session['CDOPERADOR'], $session['NRORG'], false, false, $dadosImpressao);
		if ($result['error']){
			$result['message'] = 'Ocorreu um problema na impressão da Leitura X. <br><br>' . $result['message'];
		}
		return $result;
	}

	public function getNrControlTef($CDNSUHOSTTEF) {
		$params = array(
			'CDNSUHOSTTEF' => $CDNSUHOSTTEF
		);
		$result = array(
			'error' => true,
			'message' => '',
			'data' => ''
		);

		$NRCONTROLTEF = $this->entityManager->getConnection()->fetchAssoc('GET_NRCONTROLTEF', $params);

		if(!empty($NRCONTROLTEF)) {
			if ($NRCONTROLTEF['IDSTATUSNFCE'] != 'C'){
				$result['message'] = 'Não foi possível realizar o estorno do TEF. A venda relacionada precisa estar cancelada.';
			} else {
				$result['error'] = false;
				$result['data'] = $NRCONTROLTEF;
			}
		} else {
			$result['message'] = 'Não há transações com o NSU fornecido.';
		}

		return $result;
	}

	public function ultimasVendasDesc($session, $dataFilter, $filter, $tipo) {
		$CDFILIAL = $session['CDFILIAL'];

		if($dataFilter != '%%'){
			$date = explode('/', $dataFilter);
			$day = $date[0];
			$month = $date[1];
	        $year = $date[2];
			$dataFilter = "$year-$month-$day";
			$params = array(
				'CDFILIAL' => $CDFILIAL,
				'CDCAIXA' =>  $session['CDCAIXA'],
				'DATA' => $dataFilter,
				'BUSCA' => $filter,
				'TIPO' => $tipo
			);
			$arrVendasPorNota = $this->entityManager->getConnection()->fetchAll('GET_VENDAS', $params);
		} else {
			$params = array(
				'CDFILIAL' => $CDFILIAL,
				'NRNOTAFISCALCE' => '%%',
				'CDCAIXA' => 'T'
			);
        	$arrVendasPorNota = $this->entityManager->getConnection()->fetchAll('GET_VENDAS_POR_NOTA', $params);
		}

		foreach ($arrVendasPorNota as &$venda) {
			$venda['VRTOTVENDA'] = number_format($venda['VRTOTVENDA'], 2, ',', '.');
			$venda['DTVENDA'] = date_format(date_create($venda['DTVENDA']), 'd/m/Y H:i:s');
		}
		return $arrVendasPorNota;
	}

	public function ValidaNotaPorCupom($session, $saleCode, $cdcaixa) {
		$CDFILIAL = $session['CDFILIAL'];
		if($cdcaixa=='U'){
			$cdcaixa = $session['CDCAIXA'];
			$params = array(
				'CDFILIAL' => $CDFILIAL,
				'CDCAIXA' => $session['CDCAIXA'],
				'NRORG' => $session['NRORG']
			);
			$saleCode = $this->entityManager->getConnection()->fetchAssoc('GET_ULTIMA_VENDA_FISCAL', $params);
			if(empty($saleCode)){
				throw new \Exception ("A última venda do caixa ".$session['CDCAIXA']." não foi encontrada. Por favor verifique se o caixa ".$session['CDCAIXA']." já realizou alguma venda hoje.",1);
			} else {
				$saleCode = $saleCode['NRNOTAFISCALCE'];
			}
		}
		$params = array(
			'CDFILIAL' => $CDFILIAL,
			'CDCAIXA' => $cdcaixa,
			'NRNOTAFISCALCE' => $saleCode
		);
		$arrvendasPorNota = $this->entityManager->getConnection()->fetchAll('GET_VENDAS_POR_NOTA', $params);
		foreach ($arrvendasPorNota as &$venda) {
			$venda['VRTOTVENDA'] = number_format($venda['VRTOTVENDA'], 2, ',', '.');
			$venda['DTVENDA'] = date_format(date_create($venda['DTVENDA']), 'd/m/Y H:i:s');
		}
		return $arrvendasPorNota;
	}

	public function ultimosPagementosDesc($session, $FIRST, $LAST) {
		$params = array(
			'CDFILIAL' => $session['CDFILIAL'],
			'CDCAIXA'  => $session['CDCAIXA'],
			'FIRST'    => $FIRST,
			'LAST'     => $LAST
		);

		$pagamentos = $this->entityManager->getConnection()->fetchAll('GET_PAGAMENTOS_TEF', $params);

		foreach ($pagamentos as &$pagamento) {
			$pagamento['VRMOVIVEND'] = number_format($pagamento['VRMOVIVEND'], 2, ',', '.');
			$pagamento['DTHRINCMOV'] = date_format(date_create($pagamento['DTHRINCMOV']), 'd/m/Y H:i:s');
			$pagamento['LABELGRID'] = $pagamento['NMTIPORECE'] . ' - ' . $pagamento['DTHRINCMOV'] . ' - ' . $pagamento['VRMOVIVEND'];
			$pagamento['LABELFIELD'] = $pagamento['NMTIPORECE'] . ' - ' . $pagamento['VRMOVIVEND'];
		}

		return $pagamentos;
	}

	public function getDiscountFidelity($CDFILIAL, $CDCAIXA, $CDCLIENTE, $CDCONSUMIDOR, $CDFAMILISALD, $NRSEQVENDA) {
		$params = array(
			'CDFILIAL'		=> $CDFILIAL,
			'CDCAIXA'  		=> $CDCAIXA,
			'CDCLIENTE'    	=> $CDCLIENTE,
			'CDCONSUMIDOR'  => $CDCONSUMIDOR,
			'CDFAMILISALD'  => $CDFAMILISALD,
			'NRSEQVENDA'    => $NRSEQVENDA
		);

		$FIDELITYVALUE = $this->entityManager->getConnection()->fetchAssoc('GET_DISCOUNT_FIDELITY', $params);
		return !empty($FIDELITYVALUE['VRMOVEXTCONS']) ? floatval($FIDELITYVALUE['VRMOVEXTCONS']) : 0;
	}

	public function tipoRecebimento($session) {
		$params = array(
			'CDFILIAL'   => $session['CDFILIAL'],
			'CDCAIXA'    => $session['CDCAIXA'],
			'DTABERCAIX' => null
		);


		$DTABERCAIX = $this->entityManager->getConnection()->fetchAssoc('GET_REGISTER_OPENING_DATE', $params);

		$DTABERCAIX = !empty($DTABERCAIX) ? $DTABERCAIX['DTABERCAIX'] : null;

		if ($DTABERCAIX == null) throw new \Exception ('Não foi possível encontrar a data de abertura do caixa.', 1);

		$params['DTABERCAIX'] = $this->caixaApi->convertToDateDB($DTABERCAIX);
		$types = array(
			'DTABERCAIX' => \Doctrine\DBAL\TypeS\Type::DATETIME
		);

		$result = $this->entityManager->getConnection()->fetchAll('GET_REGISTER_CLOSING_PAY_N', $params, $types);
		return $result;
	}

	public function tipoSangria() {
		return $this->entityManager->getConnection()->fetchAll('GET_TIPOSANGRIA');
	}

	public function getVenda($session, $reprintType, $NRNOTAFISCALCE){
		$CDFILIAL = $session['CDFILIAL'];
		$CDCAIXA = $session['CDCAIXA'];
		$NRORG = $session['NRORG'];
		$DTEMISSAONFCE = new \DateTime();
		$DTEMISSAONFCE = $this->databaseUtil->databaseIsOracle() ? $DTEMISSAONFCE->format('Y-m-d') : $DTEMISSAONFCE;

		$params = array(
			'CDFILIAL' => $CDFILIAL,
			'CDCAIXA' => $CDCAIXA,
			'NRORG' => $NRORG,
			'IDTPEMISVEND' =>  'N',
			'DTEMISSAONFCE' => $DTEMISSAONFCE
		);

		if ($session['IDTPEMISSAOFOS'] === 'FNC') {
			$CDSERIENFCE = $this->entityManager->getConnection()->fetchAssoc('GET_SERIE_NFCE', $params);
			$params['CDSERIENFCE'] = !empty($CDSERIENFCE['CDSERIECX']) ? $CDSERIENFCE['CDSERIECX'] : $CDSERIENFCE['CDCAIXA'];
		} else {
			$params['CDSERIENFCE'] = 'SAT';
			$params['IDTPEMISVEND'] = 'S';
		}

		$type = $this->databaseUtil->databaseIsOracle() ? array() : array('DTEMISSAONFCE' => \Doctrine\DBAL\TypeS\Type::DATETIME);

		if ($reprintType == 'U'){
			$saleCode = $this->entityManager->getConnection()->fetchAssoc('GET_ULTIMA_VENDA_FISCAL', $params, $type);
			if(empty($saleCode)){
				$venda = array(
					'error' => true,
					'message' => "A última venda do caixa ".$CDCAIXA." não foi encontrada. Verifique se o caixa ".$CDCAIXA." já realizou alguma venda."
				);
				return $venda;
			} else {
				$NRNOTAFISCALCE = !empty($saleCode['NRNOTAFISCALCE']) ? $saleCode['NRNOTAFISCALCE'] : null;
			}
		}
		$params['NRNOTAFISCALCE'] = $NRNOTAFISCALCE;
		$venda = $this->entityManager->getConnection()->fetchAssoc('GET_VENDA', $params);

		if (empty($venda)) {
			$venda = array(
				'error' => true,
				'message' => 'Operação bloqueada. Venda não encontrada.'
			);
		}

		return $venda;
	}
	public function getVRDESCCASHUDS($CDFILIAL, $CDCAIXA, $NRSEQVENDA) {
		$params = array(
			'CDFILIAL'		=> $CDFILIAL,
			'CDCAIXA'  		=> $CDCAIXA,
			'NRSEQVENDA'    => $NRSEQVENDA
		);
		$VRDESCCASHUDS = $this->entityManager->getConnection()->fetchAssoc('GET_VRDESCCASHUDS', $params);
		return !empty($VRDESCCASHUDS['VRDESCCASHUDS']) ? floatval($VRDESCCASHUDS['VRDESCCASHUDS']) : 0;
	}

	public function loadTranfers(){

		$result = $this->entityManager->getConnection()->fetchAll('GET_TRANSFERENCIAS_MESA');
		$result = array_reverse($result);
		return $result;
	}
}
